/***********************************************************************************
* Fancy Tasks: Plasmoid for fancy representing your tasks and launchers.
* Copyright (C) 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef FANCYTASKSAPPLET_HEADER
#define FANCYTASKSAPPLET_HEADER

#include <QGraphicsSceneDragDropEvent>
#include <QGraphicsSceneResizeEvent>
#include <QGraphicsLinearLayout>
#include <QDateTime>
#include <QHash>

#include <KUrl>
#include <taskmanager/task.h>
#include <taskmanager/taskitem.h>
#include <taskmanager/taskactions.h>
#include <taskmanager/taskmanager.h>
#include <taskmanager/groupmanager.h>

#include <Plasma/Applet>
#include <Plasma/FrameSvg>

#include "ui_general.h"
#include "ui_appearance.h"

class FancyTasksConfiguration;
class FancyTasksSpacer;
class FancyTasksItem;
class FancyTasksIcon;
class FancyTasksLauncher;

using TaskManager::GroupPtr;
using TaskManager::TaskPtr;
using TaskManager::StartupPtr;
using TaskManager::AbstractItemPtr;
using TaskManager::AbstractGroupableItem;
using TaskManager::GroupManager;
using TaskManager::TaskItem;
using TaskManager::TaskGroup;

class FancyTasksApplet : public Plasma::Applet
{
    Q_OBJECT

    public:
        FancyTasksApplet(QObject *parent, const QVariantList &args);

        void init();
        void dropEvent(QGraphicsSceneDragDropEvent *event);
        FancyTasksLauncher* launcherForUrl(KUrl url);
        FancyTasksConfiguration* configuration();
        FancyTasksSpacer* dropZone();
        TaskManager::GroupManager* groupManager();
        QGraphicsLinearLayout* layout();
        Plasma::FrameSvg* theme();
        QHash<FancyTasksLauncher*, FancyTasksIcon*> launcherIcons();
        QHash<AbstractItemPtr, FancyTasksIcon*> taskIcons();
        QList<FancyTasksLauncher*> launchers();
        qreal initialFactor();
        qreal iconSize();
        static QPixmap windowPreview(WId window, int size);

    protected:
        void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
        void createConfigurationInterface(KConfigDialog *parent);
        void resizeEvent(QGraphicsSceneResizeEvent *event);
        void dragMoveEvent(QGraphicsSceneDragDropEvent *event);
        void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
        void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
        void wheelEvent(QGraphicsSceneWheelEvent *event);

    public slots:
        void constraintsEvent(Plasma::Constraints constraints);
        void configAccepted();
        void saveConfig();
        void moveAnimationTypeChanged(int option);
        void showOnlyLaunchersCheckedChanged(int state);
        void itemAdded(AbstractItemPtr abstractItem);
        void itemRemoved(AbstractItemPtr abstractItem);
        void itemPositionChanged(AbstractItemPtr abstractItem);
        void addLauncher(FancyTasksLauncher *launcher, int position = -1);
        void removeLauncher(FancyTasksLauncher *launcher);
        void removeLauncherIcon(FancyTasksLauncher *launcher);
        void cleanupRemovedStartups();
        void iconHoverMoved(FancyTasksIcon *icon, qreal across);
        void iconHoverLeft();
        void moveAnimation(qreal progress);
        void showMenu();
        void showDropZone(int index);
        void updateSize();
        void updateMargins();
        void updateUrls(KUrl oldUrl, KUrl newUrl);

    private:
        QGraphicsLinearLayout *m_layout;
        GroupManager *m_groupManager;
        QList<FancyTasksLauncher*> m_launchers;
        QHash<FancyTasksLauncher*, FancyTasksIcon*> m_launcherIcons;
        QHash<AbstractItemPtr, FancyTasksIcon*> m_taskIcons;
        QHash<FancyTasksIcon*, QDateTime> m_removedStartups;
        QSize m_size;
        Plasma::FrameSvg *m_theme;
        FancyTasksConfiguration *m_configuration;
        FancyTasksSpacer *m_dropZone;
        qreal m_appletMaximumWidth;
        qreal m_appletMaximumHeight;
        qreal m_initialFactor;
        qreal m_across;
        qreal m_iconSize;
        int m_activeIcon;
        int m_animation;
        Ui::general m_generalUi;
        Ui::appearance m_appearanceUi;

    signals:
        void sizeChanged(qreal size);
};

#endif
