/***********************************************************************************
* Fancy Tasks: Plasmoid for fancy representing your tasks and launchers.
* Copyright (C) 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef FANCYTASKSICON_HEADER
#define FANCYTASKSICON_HEADER

#include <QGraphicsWidget>
#include <QGraphicsLinearLayout>
#include <QPixmap>
#include <QTimer>

#include <Plasma/Applet>

#include <KUrl>
#include <KMimeType>
#include <KPropertiesDialog>
#include <taskmanager/taskmanager.h>
#include <taskmanager/task.h>
#include <taskmanager/groupmanager.h>
#include <taskmanager/taskitem.h>
#include <taskmanager/taskactions.h>

class QPainter;
class QStyleOptionGraphicsItem;
class QRectF;
class QSizeF;
class QGraphicsSceneHoverEvent;
class FancyTasksApplet;
class FancyTasksItem;
class FancyTasksLauncher;
class FancyTasksToolTip;

using TaskManager::TaskPtr;
using TaskManager::StartupPtr;

class FancyTasksIcon : public QGraphicsWidget
{
    Q_OBJECT

    public:
        FancyTasksIcon(TaskManager::AbstractGroupableItem *abstractItem, FancyTasksLauncher *launcher, FancyTasksApplet *parent);

        FancyTasksItem* item() const;
        FancyTasksApplet* applet() const;
        qreal iconSize() const;
        qreal factor() const;
        bool isActive() const;
        QColor color() const;
        QPixmap iconPixmap(qreal size);
        QPixmap pixmap();
        QPoint popupPosition(QSize size, bool center = false);
        QRect iconGeometry();
        QRectF boundingRect() const;
        QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;

    protected:
        void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);

    public slots:
        void setFactor(qreal factor);
        void setSize(qreal size);
        void setActive(bool active);
        void setPressed(bool pressed);
        void setColor(QColor color);
        void setPosition();
        void refreshPixmap(bool force = true);
        void startAnimation(int duration, QString animation);
        void stopAnimation();
        void repeatAnimation();
        void bounceAnimation(qreal progress);
        void jumpAnimation(qreal progress);
        void blinkAnimation(qreal progress);
        void highlightAnimation(qreal progress);
        void rotateAnimation(qreal progress);
        void zoomAnimation(qreal progress);
        void reload();

    private:
        FancyTasksItem *m_item;
        QTimer *m_animationRepeater;
        QPixmap m_pixmap;
        QPixmap m_modifiedPixmap;
        QPixmap m_taskMarkPixmap;
        QPixmap m_highlightPixmap;
        QPixmap m_spotlightPixmap;
        QString m_currentAnimation;
        QColor m_color;
        QGraphicsLinearLayout *m_layout;
        FancyTasksApplet *m_applet;
        qreal m_size;
        qreal m_factor;
        qreal m_progress;
        bool m_pixmapValid;
        bool m_active;
        bool m_pressed;
        int m_currentAnimationDuration;
        int m_animation;
};

#endif
