/***********************************************************************************
* Fancy Tasks: Plasmoid for fancy representing your tasks and launchers.
* Copyright (C) 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef FANCYTASKSITEM_HEADER
#define FANCYTASKSITEM_HEADER

#include <QGraphicsWidget>
#include <QGraphicsSceneContextMenuEvent>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsSceneDragDropEvent>
#include <QPixmap>
#include <QTimer>

#include <Plasma/Applet>

#include <KUrl>
#include <KIcon>
#include <taskmanager/task.h>
#include <taskmanager/taskitem.h>
#include <taskmanager/taskactions.h>
#include <taskmanager/taskmanager.h>
#include <taskmanager/groupmanager.h>
#include <taskmanager/abstractgroupingstrategy.h>

#include "ui_group.h"

class QPainter;
class QRectF;
class QSizeF;
class QGraphicsSceneHoverEvent;
class FancyTasksApplet;
class FancyTasksIcon;
class FancyTasksLauncher;
class FancyTasksToolTip;

using TaskManager::GroupPtr;
using TaskManager::TaskPtr;
using TaskManager::StartupPtr;
using TaskManager::AbstractItemPtr;
using TaskManager::AbstractGroupableItem;
using TaskManager::GroupManager;
using TaskManager::TaskItem;
using TaskManager::TaskGroup;

class FancyTasksItem : public QGraphicsWidget
{
    Q_OBJECT

    public:
        FancyTasksItem(TaskManager::AbstractGroupableItem *abstractItem, FancyTasksLauncher *launcher, FancyTasksIcon *parent);

        enum ItemType { Other = 0, Launcher, Startup, Task, Group };

        KIcon icon();
        QPixmap thumbnail();
        QString title();
        QString description();
        bool demandsAttention();
        int itemsAmount();
        ItemType itemType();
        AbstractGroupableItem* abstractItem();
        TaskGroup* group();
        FancyTasksLauncher* launcher();

    protected:
        void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
        void hoverMoveEvent(QGraphicsSceneHoverEvent *event);
        void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
        void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
        void dragMoveEvent(QGraphicsSceneDragDropEvent *event);
        void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
        void dropEvent(QGraphicsSceneDragDropEvent *event);
        void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
        void mousePressEvent(QGraphicsSceneMouseEvent *event);
        void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
        void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);

    public slots:
        void taskChanged(::TaskManager::TaskChanges changes);
        void launcherChanged();
        void gotTaskPointer();
        void setIcon();
        void setThumbnail();
        void setItemType(ItemType itemType);
        void setTask(TaskManager::AbstractGroupableItem *abstractItem);
        void setLauncher(FancyTasksLauncher *launcher);
        void activate();
        void activateWindow(bool force = false);
        void updateToolTip();
        void updateToolTipPosition();
        void toggleToolTip();
        void blockToolTip();
        void groupDialogAccepted();
        void groupDialogClosed();
        void groupDialog();
        void groupItemAdded(AbstractItemPtr item);
        void groupItemRemoved(AbstractItemPtr item);

    private:
        ItemType m_itemType;
        TaskManager::AbstractGroupableItem *m_abstractItem;
        TaskManager::TaskItem *m_task;
        TaskManager::TaskGroup *m_group;
        FancyTasksToolTip *m_toolTip;
        FancyTasksLauncher *m_launcher;
        FancyTasksIcon *m_parent;
        KDialog *m_groupDialog;
        KIcon m_icon;
        QTimer *m_toolTipTimer;
        QTimer *m_dragTimer;
        QPixmap m_thumbnail;
        QString m_title;
        QString m_description;
        bool m_demandsAttention;
        bool m_toolTipValid;
        int m_itemsAmount;
        Ui::group m_groupUi;

    signals:
        void hoverMoved(FancyTasksIcon *icon, qreal across);
        void hoverLeft();
        void pixmapChanged();
};

#endif
