/***********************************************************************************
* Fancy Tasks: Plasmoid for fancy representing your tasks and launchers.
* Copyright (C) 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef FANCYTASKSLAUNCHER_HEADER
#define FANCYTASKSLAUNCHER_HEADER

#include <QPixmap>
#include <QGraphicsSceneDragDropEvent>

#include <Plasma/Applet>

#include <KUrl>
#include <KRun>
#include <KMenu>
#include <KIcon>
#include <KFileItem>
#include <KDesktopFile>
#include <KPropertiesDialog>
#include <kio/previewjob.h>
#include <kio/netaccess.h>
#include <kio/copyjob.h>

class FancyTasksApplet;
class FancyTasksIcon;
class FancyTasksItem;

class FancyTasksLauncher : public QObject
{
    Q_OBJECT

    public:
        FancyTasksLauncher(KUrl url, FancyTasksApplet *applet);
        ~FancyTasksLauncher();

        void dropEvent(FancyTasksIcon *parent, QGraphicsSceneDragDropEvent *event);
        void addItem(FancyTasksItem *item);
        int itemsAmount();
        bool isExecutable();
        KMenu* contextMenu();
        KIcon icon();
        KUrl url();
        QPixmap thumbnail();
        QString title();
        QString description();

    public slots:
        void setUrl(KUrl url);
        void setThumbnail(const KFileItem item, const QPixmap thumbnail);
        void setBrowseMenu();
        void openUrl(QAction *action);
        void activate();
        void remove();
        void removeItem();
        void showPropertiesDialog();
        void propertiesDialogAccepted();
        void propertiesDialogClosed();

    private:
        KMimeType::Ptr m_mimetype;
        KPropertiesDialog *m_propertiesDialog;
        KUrl m_url;
        KUrl m_realUrl;
        KIcon m_icon;
        FancyTasksApplet *m_applet;
        QPixmap m_thumbnail;
        QString m_title;
        QString m_description;
        QList<FancyTasksItem*> m_items;
        int m_itemsAmount;
        bool m_executable;

    signals:
        void urlUpdated(KUrl oldUrl, KUrl newUrl);
        void changed();
};

#endif
