/***********************************************************************************
* Fancy Tasks: Plasmoid for fancy representing your tasks and launchers.
* Copyright (C) 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef FANCYTAKSPREVIEW_HEADER
#define FANCYTAKSPREVIEW_HEADER

#include <QWidget>
#include <QSize>
#include <QMouseEvent>
#include <QDragEnterEvent>
#include <QTextDocument>

#include <Plasma/FrameSvg>

#include <KUrl>
#include <KRun>
#include <taskmanager/taskitem.h>
#include <taskmanager/taskactions.h>

using TaskManager::AbstractItemPtr;

class FancyTasksLauncher;

class FancyTasksPreview : public QWidget
{
    Q_OBJECT

    public:
        FancyTasksPreview(QPixmap pixmap, QPixmap icon, QString text, int offset, AbstractItemPtr abstractItem, QWidget *parent);
        FancyTasksPreview(QPixmap pixmap, QPixmap icon, QString text, int offset, FancyTasksLauncher *launcher, QWidget *parent);
        FancyTasksPreview(QPixmap pixmap, QPixmap icon, QString text, AbstractItemPtr abstractItem, QWidget *parent);

        TaskManager::TaskItem* task();
        Plasma::FrameSvg* background();
        QSize previewSize();
        int offset();

    protected:
        void init(QPixmap pixmap, QPixmap icon, QString text, int offset);
        void paintEvent(QPaintEvent *event);
        void enterEvent(QEvent *event);
        void leaveEvent(QEvent *event);
        void mousePressEvent(QMouseEvent *event);
        void dragEnterEvent(QDragEnterEvent *event);

    public slots:
        void updateTheme();

    private:
        Plasma::FrameSvg *m_background;
        QPixmap m_pixmap;
        QPixmap m_icon;
        QTextDocument *m_document;
        QSize m_previewSize;
        TaskManager::TaskItem *m_task;
        TaskManager::TaskGroup *m_group;
        FancyTasksLauncher* m_launcher;
        qreal m_opacity;
        int m_offset;
};

#endif
